/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>

#include "x_bim.h"
#include "x_tz.h"

#include "x_ldr_env.h"

DECLARE_GLOBAL_DATA_PTR;

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)

extern void print_bootargs_size(void);

static void setup_start_tag (bd_t *bd);

# ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags (bd_t *bd);
# endif
static void setup_commandline_tag (bd_t *bd, char *commandline);

# ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start,
			      ulong initrd_end);
# endif
static void setup_end_tag (bd_t *bd);

static struct tag *params;
#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static ulong get_sp(void);
#if defined(CONFIG_OF_LIBFDT)
static int bootm_linux_fdt(int machid, bootm_headers_t *images);
#endif

void arch_lmb_reserve(struct lmb *lmb)
{
	ulong sp;

	/*
	 * Booting a (Linux) kernel image
	 *
	 * Allocate space for command line and board info - the
	 * address should be as high as possible within the reach of
	 * the kernel (see CONFIG_SYS_BOOTMAPSZ settings), but in unused
	 * memory, which means far enough below the current stack
	 * pointer.
	 */
	sp = get_sp();
	debug("## Current stack ends at 0x%08lx ", sp);

	/* adjust sp by 1K to be safe */
	sp -= 1024;
	lmb_reserve(lmb, sp,
		    gd->bd->bi_dram[0].start + gd->bd->bi_dram[0].size - sp);
}

static void announce_and_cleanup(void)
{
	printf("\nStarting kernel ...\n\n");

#ifdef CONFIG_USB_DEVICE
	{
		extern void udc_disconnect(void);
		udc_disconnect();
	}
#endif

    {
        extern void turn_off_USB_power_switch(void);
        turn_off_USB_power_switch();
    }

	cleanup_before_linux();
}

#include "x_dram.h"
int do_bootm_linux(int flag, int argc, char *argv[], bootm_headers_t *images)
{
	bd_t	*bd = gd->bd;
	char	*s;
	int	machid = bd->bi_arch_number;
	char *emmc_log;

#ifdef CONFIG_CMD_NET
	/* Power down Ethernet NIC. */
	*(volatile unsigned long *)(0xf0032008) = 1 << 31;
#endif

#if defined(CC_TRUSTZONE_SUPPORT) || defined(CC_TRUSTZONE_64_SUPPORT)
    void (*kernel_entry)(int zero, int arch, uint params, uint kernel_entry);
#else
    void (*kernel_entry)(int zero, int arch, uint params);
#endif // CC_TRUSTZONE_SUPPORT

#ifdef CONFIG_CMDLINE_TAG
	char *commandline = getenv ("bootargs");
#endif
    emmc_log   = getenv("emmclog");

	if ((flag != 0) && (flag != BOOTM_STATE_OS_GO))
		return 1;

	s = getenv ("machid");
	if (s) {
		machid = simple_strtoul (s, NULL, 16);
		printf ("Using machid 0x%x from environment\n", machid);
	}

	show_boot_progress (15);

#ifdef CONFIG_OF_LIBFDT
	if (images->ft_len)
		return bootm_linux_fdt(machid, images);
#endif

#if defined(CC_TRUSTZONE_SUPPORT) || (CC_TRUSTZONE_64_SUPPORT)
    kernel_entry = (void *)TRUSTZONE_ENTRY;
#else
    kernel_entry = (void (*)(int, int, uint))images->ep;
#endif // CC_TRUSTZONE_SUPPORT

    debug ("## Transferring control to Linux (at address %08lx) ...\n", (ulong) kernel_entry);

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)
	setup_start_tag (bd);
#ifdef CONFIG_SERIAL_TAG
	setup_serial_tag (&params);
#endif
#ifdef CONFIG_REVISION_TAG
	setup_revision_tag (&params);
#endif
#ifdef CONFIG_SETUP_MEMORY_TAGS
	setup_memory_tags (bd);
#endif
#ifdef CONFIG_CMDLINE_TAG
	setup_commandline_tag (bd, commandline);
#endif
#ifdef CONFIG_INITRD_TAG
	if (images->rd_start && images->rd_end)
		setup_initrd_tag (bd, images->rd_start, images->rd_end);
#endif
	setup_end_tag(bd);
#endif

#ifndef TIME_TEST
	BIM_SetTimeLog(3);
#else
	//BIM_SetTimeLog(9);
#endif
	print_bootargs_size();
	announce_and_cleanup();
#ifdef CC_TRUSTZONE_64_SUPPORT
    IO_WRITE32(0, RVADDR0_L, kernel_entry);
	transform_cpu_to_64bit(0,machid,bd->bi_boot_params,images->ep, kernel_entry);
#elif defined(CC_TRUSTZONE_SUPPORT)
	kernel_entry(0, machid, bd->bi_boot_params, images->ep);
#else
	kernel_entry(0, machid, bd->bi_boot_params);
#endif
	/* does not return */

	return 1;
}

#if defined(CONFIG_OF_LIBFDT)
static int fixup_memory_node(void *blob)
{
	bd_t	*bd = gd->bd;
	
	int bank,bank_nr = CONFIG_NR_DRAM_BANKS;
#ifdef CC_TWO_MEM_REGION_BANK 
	int region, region_nr = CONFIG_NR_DRAM_REGION; 
	u64 start[CONFIG_NR_DRAM_REGION];
	u64 size[CONFIG_NR_DRAM_REGION];
#else
	u64 start[CONFIG_NR_DRAM_BANKS];
	u64 size[CONFIG_NR_DRAM_BANKS];
#endif
	char *s;
#ifdef CC_FBM_DYNMJC
	int is_support_mjc = ((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->u4SupportMJC;
#endif
#if defined(CC_DYNAMIC_FBMSRM_CONF)
		if(FBMDynIs4KMode())
		{
			s = getenv("kmem4ksize");
		}
		else
		{
			s = getenv("kmemfhdsize");
		}
#else
#ifdef CC_FBM_DYNMJC
			if(!is_support_mjc)
			{
				s = getenv("kmemsize_nomjc");
			}
			else
			{
				s = getenv("kmemsize");
			}
#else
			s = getenv("kmemsize");
#endif
#endif
			if (s)
			{
				bd->bi_dram[0].size = simple_strtoul (s, NULL, 16);
			}
#ifdef CC_TWO_MEM_REGION_BANK 
			s = getenv("kmemstart2"); 
			if (s) 
			{ 
			    bd->bi_dram[0].start2 = simple_strtoull (s, NULL, 16); 
			} 
			if(DRVCUST_GetDramSize() == DRAM_4G_CONFIG)
			{
				s = getenv("kmemsize2_4G");
			}
			else if(DRVCUST_GetDramSize() == DRAM_3_5G_CONFIG)
			{
				s = getenv("kmemsize2_3_5G");
			}
			else if(DRVCUST_GetDramSize() == DRAM_3G_CONFIG)
			{
				s = getenv("kmemsize2_3G");
			}
		#ifdef CC_MEM_SIZE_LARGE_THAN_4G
			else if(DRVCUST_GetDramSize() == DRAM_5G_CONFIG)
			{
				s = getenv("kmemsize2_5G");
			}
			else if(DRVCUST_GetDramSize() == DRAM_6G_CONFIG)
			{
				s = getenv("kmemsize2_6G");
			}
		#endif
			else
			{
				s = getenv("kmemsize2");
			} 
			if (s) 
			{ 
			    bd->bi_dram[0].size2 = simple_strtoull (s, NULL, 16); 
			} 
#endif  
#if CONFIG_NR_DRAM_BANKS>1
			s = getenv("kmem2start");
			if (s)
			{
				bd->bi_dram[1].start = simple_strtoul (s, NULL, 16);
			}
#if defined(CC_DYNAMIC_FBMSRM_CONF)
			if(FBMDynIs4KMode())
			{
				s = getenv("kmem24ksize");
			}
			else
			{
				s = getenv("kmem2fhdsize");
			}
#else
				s = getenv("kmem2size");
#endif
		
			if (s)
			{
				bd->bi_dram[1].size = simple_strtoul (s, NULL, 16);
			}

#endif

#ifdef CC_SUPPORT_CHANNEL_C
				s = getenv("kmem3start");
				if (s)
				{
					bd->bi_dram[2].start = simple_strtoul (s, NULL, 16);
				}
				s = getenv("kmem3size");
				if (s)
				{
					bd->bi_dram[2].size = simple_strtoul (s, NULL, 16);
				}
				
#ifdef CC_MT5891
					if(BSP_GetIcVersion() != IC_VER_5891_AA)
					{
							bank_nr = 3;
					}
#else
						bank_nr = 3;					
#endif
				
#endif
#ifdef CC_TWO_MEM_REGION_BANK 
	for (bank = 0, region = 0 ; bank < bank_nr; bank++, region++) {
		start[region] = bd->bi_dram[bank].start;
		size[region] = bd->bi_dram[bank].size;
		printf ("## Linux start= 0x%09llx size= 0x%09llx ...\n",  start[region], size[region]);
		#ifdef CC_TWO_MEM_REGION_BANK
		region++;
		start[region] = bd->bi_dram[bank].start2;
		size[region] = bd->bi_dram[bank].size2;
		printf ("## Linux start= 0x%09llx size= 0x%09llx ...\n",  start[region], size[region]);
		#endif
	}

	return fdt_fixup_memory_banks(blob, start, size, region_nr);
#else
		for (bank = 0; bank < bank_nr; bank++) {

		start[bank] = bd->bi_dram[bank].start;
		size[bank] = bd->bi_dram[bank].size;
		
		printf ("## Linux start= 0x%08llx size= 0x%08llx ...\n",  start[bank],size[bank]);
	}

	return fdt_fixup_memory_banks(blob, start, size, bank_nr);
#endif
}

static int bootm_linux_fdt(int machid, bootm_headers_t *images)
{
    ulong rd_len;
#if defined(CC_TRUSTZONE_SUPPORT) || defined(CC_TRUSTZONE_64_SUPPORT)
    void (*kernel_entry)(int zero, int arch, uint params, uint kernel_entry);
#else
    void (*kernel_entry)(int zero, int arch, uint params);
#endif // CC_TRUSTZONE_SUPPORT
    ulong of_size = images->ft_len;
    char **of_flat_tree = &images->ft_addr;
    ulong *initrd_start = &images->rd_start;
    ulong *initrd_end = &images->rd_end;
    struct lmb *lmb = &images->lmb;
    int ret;

#if defined(CC_TRUSTZONE_SUPPORT) || (CC_TRUSTZONE_64_SUPPORT)
    kernel_entry = (void *)TRUSTZONE_ENTRY;
#else
    kernel_entry = (void (*)(int, int, uint))images->ep;
#endif // CC_TRUSTZONE_SUPPORT

	boot_fdt_add_mem_rsv_regions(lmb, *of_flat_tree);

	rd_len = images->rd_end - images->rd_start;
	#if 0
	ret = boot_ramdisk_high(lmb, images->rd_start, rd_len,
				initrd_start, initrd_end);
	if (ret)
		return ret;
	#endif

	#ifdef CC_SUPPORT_DTO
	ret = boot_ufdt_overlay(of_flat_tree, &of_size);
	if (ret)
	{
		printf("## %s: Merge DTBO to main DTB fail!\n",__FUNCTION__);
		//return ret;
	}
	#endif
	ret = boot_relocate_fdt(lmb, of_flat_tree, &of_size);
	if (ret)
		return ret;

	printf("## Transferring control to Linux (at address %08lx) ...\n",
	       (ulong) kernel_entry);

	fdt_chosen(*of_flat_tree, 1);

	fdt_fixup_kaslr(*of_flat_tree);

	fixup_memory_node(*of_flat_tree);

	//fdt_fixup_ethernet(*of_flat_tree);

	fdt_initrd(*of_flat_tree, *initrd_start, *initrd_end, 1);

#ifndef TIME_TEST
		BIM_SetTimeLog(3);
#else
		//BIM_SetTimeLog(9);
#endif
	print_bootargs_size();
	announce_and_cleanup();
	//clear LDR_ENV_T
	memset((LDR_ENV_T *)CC_LDR_ENV_OFFSET, 0, sizeof(LDR_ENV_T));
	//run_command("fdt l /memory", 0);
	//run_command("fdt l /chosen", 0);
#ifdef CC_TRUSTZONE_64_SUPPORT
		imagesep = images->ep;
        IO_WRITE32(0, RVADDR0_L, kernel_entry);
		transform_cpu_to_64bit(*of_flat_tree,machid,0);
#elif defined(CC_TRUSTZONE_SUPPORT)
		kernel_entry(0, machid, *of_flat_tree, images->ep);
#else
		kernel_entry(0, machid, *of_flat_tree);
#endif
	/* does not return */

	return 1;
}
#endif

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)
static void setup_start_tag (bd_t *bd)
{
	params = (struct tag *) bd->bi_boot_params;

	params->hdr.tag = ATAG_CORE;
	params->hdr.size = tag_size (tag_core);

	params->u.core.flags = 0;
	params->u.core.pagesize = 0;
	params->u.core.rootdev = 0;

	params = tag_next (params);
}


#ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags (bd_t *bd)
{
	int i,bank_nr = CONFIG_NR_DRAM_BANKS;
	char *s;
#ifdef CC_FBM_DYNMJC
	int is_support_mjc = ((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->u4SupportMJC;
#endif
#if defined(CC_DYNAMIC_FBMSRM_CONF)
	if(FBMDynIs4KMode())
	{
		s = getenv("kmem4ksize");
	}
	else
	{
	    s = getenv("kmemfhdsize");
	}
#else
    #ifdef CC_FBM_DYNMJC
    if(!is_support_mjc)
    {
        s = getenv("kmemsize_nomjc");
    }
    else
    {
        s = getenv("kmemsize");
    }
    #else
    s = getenv("kmemsize");
    #endif
#endif
	if (s)
	{
		bd->bi_dram[0].size = simple_strtoul (s, NULL, 16);
	}
#ifdef CC_TWO_MEM_REGION_BANK
    s = getenv("kmemstart2");
    if (s)
    {
        bd->bi_dram[0].start2 = simple_strtoull (s, NULL, 16);
    }
    if(DRVCUST_GetDramSize() == DRAM_4G_CONFIG)
	{
		s = getenv("kmemsize2_4G");
	}
	else if(DRVCUST_GetDramSize() == DRAM_3_5G_CONFIG)
	{
		s = getenv("kmemsize2_3_5G");
	}
	else if(DRVCUST_GetDramSize() == DRAM_3G_CONFIG)
	{
		s = getenv("kmemsize2_3G");
	}
#ifdef CC_MEM_SIZE_LARGE_THAN_4G
	else if(DRVCUST_GetDramSize() == DRAM_5G_CONFIG)
	{
		s = getenv("kmemsize2_5G");
	}
	else if(DRVCUST_GetDramSize() == DRAM_6G_CONFIG)
	{
		s = getenv("kmemsize2_6G");
	}
#endif

	else
	{
    	s = getenv("kmemsize2");
	}
    if (s)
    {
        bd->bi_dram[0].size2 = simple_strtoull (s, NULL, 16);
    }
#endif
#if CONFIG_NR_DRAM_BANKS>1
	s = getenv("kmem2start");
	if (s)
	{
		bd->bi_dram[1].start = simple_strtoul (s, NULL, 16);
	}
#if defined(CC_DYNAMIC_FBMSRM_CONF)
	if(FBMDynIs4KMode())
	{
		s = getenv("kmem24ksize");
	}
	else
	{
		s = getenv("kmem2fhdsize");
	}
#else
	s = getenv("kmem2size");
#endif
	if (s)
	{
		bd->bi_dram[1].size = simple_strtoul (s, NULL, 16);
	}
#endif
#ifdef CC_SUPPORT_CHANNEL_C
	s = getenv("kmem3start");
	if (s)
	{
		bd->bi_dram[2].start = simple_strtoul (s, NULL, 16);
	}
	s = getenv("kmem3size");
	if (s)
	{
		bd->bi_dram[2].size = simple_strtoul (s, NULL, 16);
	}
#ifdef CC_MT5891
	if(BSP_GetIcVersion() != IC_VER_5891_AA)
	{
		bank_nr = 3;
	}
#else
	bank_nr = 3;
#endif
#endif
	for (i = 0; i < bank_nr; i++) {
		params->hdr.tag = ATAG_MEM;
		params->hdr.size = tag_size (tag_mem32);

		params->u.mem.start = bd->bi_dram[i].start;
		params->u.mem.size = bd->bi_dram[i].size;

		params = tag_next (params);
#ifdef CC_TWO_MEM_REGION_BANK
		params->hdr.tag = ATAG_MEM;
		params->hdr.size = tag_size (tag_mem32);
		
		if(bd->bi_dram[i].start2 > 0xffffffff || bd->bi_dram[i].size2 > 0xffffffff)
		{
			printf("## Error, start or size of DRAM bank%d segment2 is too large!\n",i);
		}

		params->u.mem.start = bd->bi_dram[i].start2;
		params->u.mem.size = bd->bi_dram[i].size2;

		params = tag_next (params);
#endif
	}
}
#endif /* CONFIG_SETUP_MEMORY_TAGS */


static void setup_commandline_tag (bd_t *bd, char *commandline)
{
	char *p;

	if (!commandline)
		return;

	/* eat leading white space */
	for (p = commandline; *p == ' '; p++);

	/* skip non-existent command lines so the kernel will still
	 * use its default command line.
	 */
	if (*p == '\0')
		return;

	params->hdr.tag = ATAG_CMDLINE;
	params->hdr.size =
		(sizeof (struct tag_header) + strlen (p) + 1 + 4) >> 2;

	strcpy (params->u.cmdline.cmdline, p);

	params = tag_next (params);
}


#ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start, ulong initrd_end)
{
	/* an ATAG_INITRD node tells the kernel where the compressed
	 * ramdisk can be found. ATAG_RDIMG is a better name, actually.
	 */
	params->hdr.tag = ATAG_INITRD2;
	params->hdr.size = tag_size (tag_initrd);

	params->u.initrd.start = initrd_start;
	params->u.initrd.size = initrd_end - initrd_start;

	params = tag_next (params);
}
#endif /* CONFIG_INITRD_TAG */

#ifdef CONFIG_SERIAL_TAG
void setup_serial_tag (struct tag **tmp)
{
	struct tag *params = *tmp;
	struct tag_serialnr serialnr;
	void get_board_serial(struct tag_serialnr *serialnr);

	get_board_serial(&serialnr);
	params->hdr.tag = ATAG_SERIAL;
	params->hdr.size = tag_size (tag_serialnr);
	params->u.serialnr.low = serialnr.low;
	params->u.serialnr.high= serialnr.high;
	params = tag_next (params);
	*tmp = params;
}
#endif

#ifdef CONFIG_REVISION_TAG
void setup_revision_tag(struct tag **in_params)
{
	u32 rev = 0;
	u32 get_board_rev(void);

	rev = get_board_rev();
	params->hdr.tag = ATAG_REVISION;
	params->hdr.size = tag_size (tag_revision);
	params->u.revision.rev = rev;
	params = tag_next (params);
}
#endif  /* CONFIG_REVISION_TAG */

static void setup_end_tag (bd_t *bd)
{
	params->hdr.tag = ATAG_NONE;
	params->hdr.size = 0;
}
#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static ulong get_sp(void)
{
	ulong ret;

	asm("mov %0, sp" : "=r"(ret) : );
	return ret;
}
